# Generated by roxygen2: do not edit by hand

S3method(print,dnaEPICO_dnamReport)
export(analyzeSvaEnmix)
export(annotateMethylationGLMSummaries)
export(annotateMethylationLMESummaries)
export(assessSamplesMinfiEwasWater)
export(buildClockFoundationInputsPreprocessingPheno)
export(buildRawMinfiEwasWater)
export(collectSignificantCpGsMethylationGLM)
export(collectSignificantInteractionsMethylationLME)
export(combineTimepointsPreprocessingPheno)
export(dnamReport)
export(estimateLC)
export(estimateLCMinfiEwasWater)
export(estimateSvaEnmixControls)
export(extractMake)
export(extractMetricsMinfiEwasWater)
export(filterProbesMinfiEwasWater)
export(filterSamplesMinfiEwasWater)
export(fitMethylationGLMModels)
export(fitMethylationLMEModels)
export(loadMetricsPreprocessingPheno)
export(mergeSvaTargetsEnmix)
export(methylationGLM)
export(methylationLME)
export(normalizeMinfiEwasWater)
export(plotAssessmentMinfiEwasWater)
export(plotCtrlMinfiEwasWater)
export(plotMethylationGLMDiagnostics)
export(plotMethylationGLMDistributions)
export(plotMethylationLMEDiagnostics)
export(plotMetricsMinfiEwasWater)
export(plotNormalizationMinfiEwasWater)
export(plotRawDensityMinfiEwasWater)
export(plotSexMinfiEwasWater)
export(plotSvaEnmix)
export(predictSexMinfiEwasWater)
export(prepareDnamReportInputs)
export(prepareMethylationGLMData)
export(prepareMethylationLMEData)
export(preprocessingMinfiEwasWater)
export(preprocessingPheno)
export(readPhenotypeTargets)
export(readRGSetMinfiEwasWater)
export(renderDnamReport)
export(splitTimepointsPreprocessingPheno)
export(summarizeMethylationGLMModels)
export(summarizeMethylationLMEModels)
export(summarizeTimepointsMethylationLME)
export(svaEnmix)
export(writeMethylationGLMOutputs)
export(writeMethylationLMEOutputs)
export(writePhenoLCMinfiEwasWater)
export(writePreprocessingPhenoOutputs)
export(writeSvaEnmixOutputs)
