% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnamReport.R
\name{renderDnamReport}
\alias{renderDnamReport}
\title{Render a prepared DNA methylation report}
\usage{
renderDnamReport(
  preparedReport,
  verbose = FALSE,
  logs = FALSE,
  logDir = NULL,
  clean = TRUE
)
}
\arguments{
\item{preparedReport}{Object returned by \code{prepareDnamReportInputs()}.}

\item{verbose}{Logical. If \code{TRUE}, emit progress messages.}

\item{logs}{Logical. If \code{TRUE}, write progress messages to a log file.}

\item{logDir}{Character or \code{NULL}. Directory for optional log files.}

\item{clean}{Logical. Retained for backwards compatibility.}
}
\value{
A list with class \code{'dnaEPICO_dnamReport_render'}.
}
\description{
Render a prepared DNA methylation report
}
\examples{
report_root <- file.path(tempdir(), "dnaepico-render-example")
prepared <- prepareDnamReportInputs(
    outputDir = file.path(report_root, "reports")
)
rendered <- renderDnamReport(prepared)
rendered$status

}
