% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnamReport.R
\name{print.dnaEPICO_dnamReport}
\alias{print.dnaEPICO_dnamReport}
\title{Print a DNA methylation report result}
\usage{
\method{print}{dnaEPICO_dnamReport}(x, ...)
}
\arguments{
\item{x}{Object returned by \code{\link[=dnamReport]{dnamReport()}}.}

\item{...}{Additional arguments ignored.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a DNA methylation report result
}
