% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/result_classes.R
\docType{class}
\name{dnaEPICO_preprocessingMinfiEwasWater-class}
\alias{dnaEPICO_preprocessingMinfiEwasWater-class}
\alias{dnaEPICO_preprocessingMinfiEwasWater}
\title{Result class returned by preprocessingMinfiEwasWater}
\description{
Objects of class \code{'dnaEPICO_preprocessingMinfiEwasWater'} are list-based
results returned by \code{\link[=preprocessingMinfiEwasWater]{preprocessingMinfiEwasWater()}}. They are lightweight
S3-style containers rather than formal S4 classes.
}
\section{Structure}{

\describe{
\item{targets}{Filtered phenotype table aligned to the retained samples.}
\item{RGSet}{Filtered \code{RGChannelSet} used in downstream preprocessing.}
\item{rawData}{Object returned by \code{\link[=buildRawMinfiEwasWater]{buildRawMinfiEwasWater()}}.}
\item{assessment}{Object returned by \code{\link[=assessSamplesMinfiEwasWater]{assessSamplesMinfiEwasWater()}}.}
\item{sexData}{Object returned by \code{\link[=predictSexMinfiEwasWater]{predictSexMinfiEwasWater()}}.}
\item{normData}{Object returned by \code{\link[=normalizeMinfiEwasWater]{normalizeMinfiEwasWater()}}.}
\item{filterData}{Object returned by \code{\link[=filterProbesMinfiEwasWater]{filterProbesMinfiEwasWater()}}.}
\item{metricsData}{Object returned by \code{\link[=extractMetricsMinfiEwasWater]{extractMetricsMinfiEwasWater()}}.}
\item{lcData}{Object returned by \code{\link[=estimateLCMinfiEwasWater]{estimateLCMinfiEwasWater()}}.}
\item{logFile}{Resolved path to the optional log file, or \code{NULL} when
logging was disabled.}
}
}

\seealso{
\code{\link[=preprocessingMinfiEwasWater]{preprocessingMinfiEwasWater()}}
}
