% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/result_classes.R
\docType{class}
\name{dnaEPICO_methylationGLM-class}
\alias{dnaEPICO_methylationGLM-class}
\alias{dnaEPICO_methylationGLM}
\title{Result class returned by methylationGLM}
\description{
Objects of class \code{'dnaEPICO_methylationGLM'} are list-based results
returned by \code{\link[=methylationGLM]{methylationGLM()}}. They collect the prepared analysis table,
fitted models, summaries, diagnostics, annotations, and optional saved files.
}
\section{Structure}{

\describe{
\item{preparedData}{Object returned by \code{\link[=prepareMethylationGLMData]{prepareMethylationGLMData()}}.}
\item{distributionPlots}{Object returned by
\code{\link[=plotMethylationGLMDistributions]{plotMethylationGLMDistributions()}}.}
\item{modelFits}{Object returned by \code{\link[=fitMethylationGLMModels]{fitMethylationGLMModels()}}.}
\item{modelSummaries}{Object returned by
\code{\link[=summarizeMethylationGLMModels]{summarizeMethylationGLMModels()}}.}
\item{significantCpGs}{Object returned by
\code{\link[=collectSignificantCpGsMethylationGLM]{collectSignificantCpGsMethylationGLM()}}.}
\item{diagnosticPlots}{Object returned by
\code{\link[=plotMethylationGLMDiagnostics]{plotMethylationGLMDiagnostics()}}.}
\item{annotation}{Object returned by
\code{\link[=annotateMethylationGLMSummaries]{annotateMethylationGLMSummaries()}}.}
\item{savedFiles}{Object returned by \code{\link[=writeMethylationGLMOutputs]{writeMethylationGLMOutputs()}} when
\code{saveOutputs = TRUE}, otherwise \code{NULL}.}
}
}

\seealso{
\code{\link[=methylationGLM]{methylationGLM()}}
}
